#Open Autism Software. http://www.openautismsoftware.org
#This software is provided under the GNU General Public License, version 3
#See the license at http://www.gnu.org/licenses/gpl.html
#Authors: Thomas Hansen

from __future__ import with_statement

# PYMT Plugin integration
IS_PYMT_PLUGIN = True
PLUGIN_TITLE = 'Untangle Game'
PLUGIN_AUTHOR = 'Thomas Hansen'
PLUGIN_EMAIL = 'thomas.hansen@gmail.com'
PLUGIN_DESCRIPTION = 'Untangle game !'

from pymt import *
from graph import *
from OpenGL.GL import *
from OpenGL.GLU import *

import time


class NewGameMenu(MTAnchorLayout):
    def __init__(self, **kwargs):

        super(NewGameMenu, self).__init__(**kwargs)
        self.window = getWindow()
        self.trial_num = 0

        menu = MTBoxLayout(spacing=50, orientation='vertical', size_hint=(None, None))

        box = MTBoxLayout()

        b1 = MTButton(label="10 Vertices", size=(200,100))
        b1.push_handlers(on_release=curry(self.startNewGame, 10))
        box.add_widget(b1)

        b2 = MTButton(label="15 Vertices", size=(200,100))
        b2.push_handlers(on_release=curry(self.startNewGame, 15))
        box.add_widget(b2)

        b3 = MTButton(label="20 Vertices", size=(200,100))
        b3.push_handlers(on_release=curry(self.startNewGame, 20))
        box.add_widget(b3)

        b4 = MTButton(label="25 Vertices", size=(200,100))
        b4.push_handlers(on_release=curry(self.startNewGame, 25))
        box.add_widget(b4)

        menu.add_widget(box)



        self.split_button = MTToggleButton(label="Split Groups: No", size=(800,100))
        def toggle(touch):
            if self.split_button.label == "Split Groups: Yes":
                self.split_button.label = "Split Groups: No"
            else:
                self.split_button.label = "Split Groups: Yes"
        self.split_button.push_handlers(on_release=toggle)
        menu.add_widget(self.split_button)

        self.add_widget(menu)

        self.graph = None
        self.start_time = None
        self.stop_time = None


    def draw(self):
        if self.start_time and self.stop_time:
            set_color(0,0,0,0.8)
            drawRectangle(size=self.window.size)

            duration = str(self.stop_time - self.start_time)[:4] + " sec"
            glColor4f(0.5,1,0.5,1)
            drawLabel("Untangled!", pos=(self.center[0], self.center[1]+340), font_size=64)
            glColor4f(0.7,0.7,0.7,1)
            drawLabel("time: "+duration+"  moves: "+str(self.num_moves), pos=(self.center[0], self.center[1]+250), font_size=50)

    def startNewGame(self, numVerts, *largs):
        if self.graph:
            self.window.remove_widget(self.graph)
        self.graph = GraphUI(size=numVerts, w=self.window, menu=self, split=self.split_button.label == "Split Groups: Yes")

        self.window.add_widget(self.graph)

        self.trial_num += 1


        self.window.remove_widget(self)
        self.start_time = time.time()


class GraphUI(MTWidget):
    def __init__(self, size=15, w=None, menu=None, split=False):
        MTWidget.__init__(self)
        self.menu = menu
        self.g = Graph(size,displaySize=w.size, split=split)
        self.touch2vertex = {}
        self.num_moves = 0
        self.done = False
        self.num_moves_since_check = 0 #if we try to solve on every move event things get slow

    def draw(self):
        self.g.draw()
        if not self.done:
            w = self.get_parent_window()
            glColor4f(0.7,0.7,0.7,1)
            duration = time.time() - self.menu.start_time
            label = 'Time: %4.1f - Moves: %d' % (duration, self.num_moves)
            drawLabel(label, pos=(w.center[0], 30), font_size=30)

    def on_touch_down(self, touch):
        if self.done:
            return
        touchedVertex = self.g.collideVerts(touch.x,touch.y)
        if touchedVertex: self.touch2vertex[touch.id] = touchedVertex

    def on_touch_up(self, touch):
        if self.done:
            return
        self.num_moves +=1
        if self.touch2vertex.has_key(touch.id):
            del self.touch2vertex[touch.id]
        if self.g.is_solved():
            #self.g = Graph(15,displaySize=w.size)
            self.done = True
            self.menu.stop_time =  time.time()
            self.menu.num_moves = self.num_moves
            self.parent.add_widget(self.menu)

    def on_touch_move(self, touch):
        if self.done:
            return
        if self.touch2vertex.has_key(touch.id):
                    self.touch2vertex[touch.id][0] = touch.x
                    self.touch2vertex[touch.id][1] = touch.y
        self.num_moves_since_check += 1
        if self.num_moves_since_check%4 == 0:
            #self.g.is_solved()
            self.num_moves_since_check = 0






runTouchApp(NewGameMenu())
